@@ -19,6 +19,7 @@ import pyutilib.services
 from coopr.opt.base import *
 from coopr.plugins.converter.pico import PicoMIPConverter
 
+from six import iteritems
 
 class PyomoMIPConverter(ManagedSingletonPlugin):
 
@@ -55,7 +56,13 @@ class PyomoMIPConverter(ManagedSingletonPlugin):
         import coopr.pyomo
 
         capabilities = kwds.pop("capabilities", None)
-        symbolic_solver_labels = kwds.pop('symbolic_solver_labels',False)
+
+        # all non-consumed keywords are assumed to be options 
+        # that should be passed to the writer.
+        io_options = {}
+        for kwd, value in iteritems(kwds):
+            io_options[kwd] = value
+        kwds.clear()
 
         if isinstance(args[2], basestring):
             instance = None
@@ -68,7 +75,7 @@ class PyomoMIPConverter(ManagedSingletonPlugin):
                 (problem_filename, varmap) = instance.write(filename=problem_filename,
                                                             format=ProblemFormat.cpxlp,
                                                             solver_capability=capabilities,
-                                                            symbolic_solver_labels=symbolic_solver_labels)
+                                                            io_options=io_options)
                 return (problem_filename,),varmap # no map file is necessary
             else:
                 ans = coopr.pyomo.scripting.pyomo2lp(['--save-model',problem_filename,args[2]])
@@ -85,7 +92,7 @@ class PyomoMIPConverter(ManagedSingletonPlugin):
                 (problem_filename, varmap) = instance.write(filename=problem_filename, 
                                                             format=ProblemFormat.nl, 
                                                             solver_capability=capabilities,
-                                                            symbolic_solver_labels=symbolic_solver_labels)
+                                                            io_options=io_options)
             else:
                 ans = coopr.pyomo.scripting.pyomo2nl(['--save-model',problem_filename,args[2]])
                 if ans.errorcode:
@@ -112,7 +119,7 @@ class PyomoMIPConverter(ManagedSingletonPlugin):
                 (problem_filename, varmap) = instance.write(filename=problem_filename,
                                                             format=ProblemFormat.osil,
                                                             solver_capability=capabilities,
-                                                            symbolic_solver_labels=symbolic_solver_labels)
+                                                            io_options=io_options)
                 return (problem_filename,), None
             else:
                 msg = 'There is currently no script conversion available from\n'
