@@ -11,7 +11,7 @@
 import pyutilib.component.core
 pyutilib.component.core.PluginGlobals.push_env( 'coopr.opt' )
 
-import pyro
-import phpyro
+import coopr.plugins.smanager.pyro
+import coopr.plugins.smanager.phpyro
 
 pyutilib.component.core.PluginGlobals.pop_env()
