@@ -46,17 +46,57 @@ PHSolverServerAction = Enum(
 
 class SolverManager_PHPyro(AsynchronousSolverManager):
 
-    pyutilib.component.core.alias('phpyro')
+    pyutilib.component.core.alias('phpyro', doc="Specialized PH solver manager that uses pyro")
 
     def clear(self):
         """
         Clear manager state
         """
+
         AsynchronousSolverManager.clear(self)
+
+        # the client-side interface to the dispatch server.
         self.client = pyutilib.pyro.Client()
+
+        # only useful for debugging communication patterns - results
+        # in a ton of output.
         self._verbose = False
+
+        # map from task id to the corresponding action handle.
+        # we only retain entries for tasks for which we expect
+        # a result/response.
         self._ah = {}
 
+        # the list of cached results obtained from the dispatch server.
+        # to avoid communication overhead, grab any/all results available,
+        # and then cache them here - but return one-at-a-time via
+        # the standard _perform_wait_any interface. the elements in this
+        # list are simply tasks - at this point, we don't care about the
+        # queue name associated with the task.
+        self._results_waiting = []
+
+    #
+    # a utility to extract a single result from the _results_waiting list.
+    #
+
+    def _extract_result(self):
+
+        if len(self._results_waiting) == 0:
+            raise RuntimeError("There are no results available for extraction from the PHPyro solver manager - call to _extract_result is not valid.")
+        
+        task = self._results_waiting.pop(0)
+
+        if task.id in self._ah:
+            ah = self._ah[task.id]
+            self._ah[task.id] = None
+            ah.status = ActionStatus.done
+            # TBD - what is the 'results' object - can we just load results directly into there?
+            self.results[ah.id] = pickle.loads(task.result)
+            return ah
+        else:
+            # if we are here, this is really bad news!
+            raise RuntimeError("The PHPyro solver manager found results for task with id=" + str(task.id) + " - but no corresponding action handle could be located!")
+
     def _perform_queue(self, ah, *args, **kwds):
         """
         Perform the queue operation.  This method returns the ActionHandle,
@@ -80,16 +120,24 @@ class SolverManager_PHPyro(AsynchronousSolverManager):
             # we always want to pass a verbose flag to the solver server.
             kwds["verbose"] = False 
 
+        if "generateResponse" in kwds:
+            generateResponse = kwds.pop("generateResponse")
+        else:
+            generateResponse = True
+
         #
-        # Pickl everything into one big data object via the "Bunch" command and post the task.
+        # Pickle everything into one big data object via the "Bunch" command and post the task.
         #
-        data=pyutilib.misc.Bunch(**kwds)
+        data = pyutilib.misc.Bunch(**kwds)
 
         # NOTE: the task type (type=) should be the name of the scenario/bundle!
 
-        task = pyutilib.pyro.Task(data=data, id=ah.id)
+        task = pyutilib.pyro.Task(data=data, id=ah.id, generateResponse=generateResponse)
         self.client.add_task(task, verbose=self._verbose, override_type=instance_name)
-        self._ah[task.id] = ah
+
+        # only populate the action_handle-to-task dictionary is a response is expected.
+        if generateResponse is True:
+            self._ah[task.id] = ah
 
         return ah
 
@@ -102,26 +150,17 @@ class SolverManager_PHPyro(AsynchronousSolverManager):
         to indicate an error.
         """
 
-        # TBD: ONLY REALLY NEED TO RETURN ANY ONE - MODIFY DISPATCHER TO RETURN THE FIRST OF ANY QUEUE THAT HAS SOMETHING.
-        queues_to_check = self.client.queues_with_results()
-        if len(queues_to_check) > 0:
+        if len(self._results_waiting) > 0:
+            return self._extract_result()
+            
+        elif len(self.client.queues_with_results()) > 0:
 
-           # this protects us against the case where we get an 
-           # action handle that we didn't know about or expect.
+            all_results = self.client.get_results_all_queues()
 
-           for queue_name in queues_to_check:
-
-               task = self.client.get_result(override_type=queue_name)
-               if task.id in self._ah:
-                   ah = self._ah[task.id]
-                   self._ah[task.id] = None
-                   ah.status = ActionStatus.done
-                   self.results[ah.id] = pickle.loads(task.result)
-                   return ah
-               else:
-                   # TBD: WE SHOULD OBVIOUSLY DO SOMETHING ELSE HERE.
-                   print("TROUBLE - GOT RESULTS WE DIDN'T EXPECT!!!")
+            for task in all_results:
+                self._results_waiting.append(task)                
         else:
+
           # if the queues are all empty, wait some time for things to fill up.
           # constantly pinging dispatch servers wastes their time, and inhibits
           # task worker communication. the good thing about queues_to_check
