@@ -36,7 +36,7 @@ class ASL(SystemCallSolver):
     """A generic optimizer that uses the AMPL Solver Library to interface with applications.
     """
 
-    pyutilib.component.core.alias('_asl', doc='Shell interface to a generic optimizer that uses the AMPL Solver Library to interface with application')
+    pyutilib.component.core.alias('asl', doc='Interface for solvers using the AMPL Solver Library')
 
     def __init__(self, **kwds):
         #
@@ -52,8 +52,9 @@ class ASL(SystemCallSolver):
         self._valid_result_formats = {}
         self._valid_result_formats[ProblemFormat.nl] = [ResultsFormat.sol]
         self.set_problem_format(ProblemFormat.nl)
-        
+        #
         # Note: Undefined capabilities default to 'None'
+        #
         self._capabilities = pyutilib.misc.Options()
         self._capabilities.linear = True
         self._capabilities.integer = True
@@ -61,6 +62,10 @@ class ASL(SystemCallSolver):
         self._capabilities.quadratic_constraint = True
         self._capabilities.sos1 = True
         self._capabilities.sos2 = True
+        #
+        # This interface is always availble
+        #
+        self._assert_available = True
 
     def _default_results_format(self, prob_format):
         return ResultsFormat.sol
