@@ -463,7 +463,7 @@ class CBCSHELL(SystemCallSolver):
                 constraint_ax = eval(tokens[2]) # CBC reports the constraint row times the solution vector - not the slack.
                 constraint_dual = eval(tokens[3])
                 if constraint.startswith('c_'):
-                    solution.constraint[constraint].dual = constraint_dual
+                    solution.constraint[constraint] = {"Dual" : constraint_dual, "Id" : len(solution.constraint)}
                 elif constraint.startswith('r_l_'):
                     range_duals.setdefault(constraint[4:],[0,0])[0] = constraint_dual
                 elif constraint.startswith('r_u_'):
@@ -494,13 +494,13 @@ class CBCSHELL(SystemCallSolver):
             INPUT.close()
 
         # For the range constraints, supply only the dual with the largest
-        # magnitude (at least one should always be numericly zero)
-        scon = solution.Constraint
+        # magnitude (at least one should always be numerically zero)
+        soln_constraints = solution.constraint
         for key,(ld,ud) in iteritems(range_duals):
             if abs(ld) > abs(ud):
-                scon['r_l_'+key].dual = ld
+                soln_constraints['r_l_'+key] = {"Dual" : ld, "Id" : len(soln_constraints)}
             else:
-                scon['r_u_'+key].dual = ud
+                soln_constraints['r_u_'+key] = {"Dual" : ud, "Id" : len(soln_constraints)}
 
 
 class MockCBC(CBCSHELL,MockMIP):
