@@ -63,7 +63,7 @@ class CPLEX(OptSolver):
         if mode == 'os':
             opt = SolverFactory('_ossolver', **kwds)
         elif mode == 'nl':
-            opt = SolverFactory('_asl', **kwds)
+            opt = SolverFactory('asl', **kwds)
         else:
             logging.getLogger('coopr.plugins').error('Unknown IO type: %s' % mode)
             return
@@ -355,6 +355,9 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
                 # technically, I'm not sure if this is CPLEX user time or user+system - CPLEX doesn't appear
                 # to differentiate, and I'm not sure we can always provide a break-down.
                 results.solver.user_time = float(tokens[3])
+            elif len(tokens) >= 4 and tokens[0] == "Primal" and tokens[1] == "simplex" and tokens[3] == "Optimal:":
+                results.solver.termination_condition = TerminationCondition.optimal
+                results.solver.termination_message = ' '.join(tokens)
             elif len(tokens) >= 4 and tokens[0] == "Dual" and tokens[1] == "simplex" and tokens[3] == "Optimal:":
                 results.solver.termination_condition = TerminationCondition.optimal
                 results.solver.termination_message = ' '.join(tokens)
@@ -456,6 +459,9 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
             if not flag:
                 raise RuntimeError("***The CPLEX solver plugin cannot extract solution suffix="+suffix)
 
+        # check for existence of the solution file
+        # not sure why we just return - would think that we
+        # would want to indicate some sort of error
         if not os.path.exists(self.soln_file):
             return
 
@@ -463,7 +469,11 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
         range_slacks = {}
         soln = Solution()
         soln.objective['__default_objective__'].value=None
-        soln_variable = soln.variable # caching for efficiency
+
+        # caching for efficiency
+        soln_variables = soln.variable 
+        soln_constraints = soln.constraint 
+
         INPUT = open(self.soln_file,"r")
         results.problem.number_of_objectives=1
         time_limit_exceeded = False
@@ -493,7 +503,7 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
 
                 # skip the "constant-one" variable, used to capture/retain objective offsets in the CPLEX LP format.
                 if variable_name != "ONE_VAR_CONSTANT":
-                    variable = soln_variable[variable_name] = {"Value" : float(variable_value), "Id" : len(soln_variable)}
+                    variable = soln_variables[variable_name] = {"Value" : float(variable_value), "Id" : len(soln_variables)}
                     if (variable_reduced_cost is not None) and (extract_reduced_costs is True):
                         try:
                             if extract_rc is True:
@@ -512,7 +522,7 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
                         except:
                             raise ValueError("Unexpected reduced-cost value="+str(variable_reduced_cost)+" encountered for variable="+variable_name)
             elif (tokens[0] == "constraint") and ((extract_duals is True) or (extract_slacks is True)):
-                range = False
+                is_range = False
                 rlabel = None
                 rkey = None
                 for i in xrange(1,len(tokens)):
@@ -520,23 +530,23 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
                     field_value = tokens[i].split('=')[1].lstrip("\"").rstrip("\"")
                     if field_name == "name":
                         if field_value.startswith('c_'):
-                            constraint = soln.constraint[field_value]
+                            constraint = soln_constraints[field_value] = {"Id" : len(soln_constraints)}
                         elif field_value.startswith('r_l_'):
-                            range = True
+                            is_range = True
                             rlabel = field_value[4:]
                             rkey = 0
                         elif field_value.startswith('r_u_'):
-                            range = True
+                            is_range = True
                             rlabel = field_value[4:]
                             rkey = 1
                     elif (extract_duals is True) and (field_name == "dual"): # for LPs
-                        if range is False:
-                            constraint.dual = float(field_value)
+                        if is_range is False:
+                            constraint["Dual"] = float(field_value)
                         else:
                             range_duals.setdefault(rlabel,[0,0])[rkey] = float(field_value)
                     elif (extract_slacks is True) and (field_name == "slack"): # for MIPs
-                        if range is False:
-                            constraint.slack = float(field_value)
+                        if is_range is False:
+                            constraint["Slack"] = float(field_value)
                         else:
                             range_slacks.setdefault(rlabel,[0,0])[rkey] = float(field_value)
             elif tokens[0].startswith("problemName"):
@@ -624,19 +634,18 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
             soln.gap = self._gap
 
         # For the range constraints, supply only the dual with the largest
-        # magnitude (at least one should always be numericly zero)
-        scon = soln.Constraint
+        # magnitude (at least one should always be numerically zero)
         for key,(ld,ud) in iteritems(range_duals):
             if abs(ld) > abs(ud):
-                scon['r_l_'+key].dual = ld
+                soln_constraints['r_l_'+key] = {"Dual" : ld, "Id" : len(soln_constraints)}
             else:
-                scon['r_u_'+key].dual = ud
+                soln_constraints['r_u_'+key] = {"Dual" : ud, "Id" : len(soln_constraints)}
         # slacks
         for key,(ls,us) in iteritems(range_slacks):
             if abs(ls) > abs(us):
-                scon['r_l_'+key].slack = ls
+                soln_constraints.setdefault('r_l_'+key,{"Id" : len(soln_constraints)})["Slack"] = ls
             else:
-                scon['r_u_'+key].slack = us
+                soln_constraints.setdefault('r_u_'+key,{"Id" : len(soln_constraints)})["Slack"] = us
 
         if not results.solver.status is SolverStatus.error:
             if results.solver.termination_condition in [TerminationCondition.unknown,
