@@ -41,51 +41,59 @@ from coopr.opt.base import *
 from coopr.opt.base.solvers import _extract_version
 from coopr.opt.results import *
 from coopr.opt.solver import *
-from coopr.pyomo.base import SymbolMap, NumericLabeler, Suffix
+from coopr.pyomo.base import SymbolMap, NumericLabeler, Suffix, TextLabeler
 from coopr.pyomo.base.numvalue import value
 from coopr.pyomo.base.block import active_subcomponents_generator, active_subcomponents_data_generator
+from coopr.plugins.solvers import wrappers
 
 try:
     unicode
 except:
     basestring = str
 
+
+class CplexSolverWrapper(wrappers.MIPSolverWrapper):
+
+    def __init__(self, solver):
+        self.cplex = solver
+
+    def add(self, constraint):
+        """TODO"""
+        pass
+
+
 class ModelSOS(object):
     def __init__(self):
         self.sosType = {}
         self.sosName = {}
         self.varnames = {}
+        self.varids = {}
         self.weights = {}
         self.block_cntr = 0
 
-    def count_constraint(self,symbol_map,labeler,con,name,level,index=None):
+    def count_constraint(self,symbol_map,labeler,soscondata):
+
+        Vars = soscondata.get_members()
+        Weights = soscondata.get_weights()
+        Level = soscondata.get_level()
 
         self.block_cntr += 1
-        self.varnames[self.block_cntr] = []
-        self.weights[self.block_cntr] = []
-        if level == 1:
+        varnames = self.varnames[self.block_cntr] = []
+        varids = self.varids[self.block_cntr] = []
+        weights = self.weights[self.block_cntr] = []
+        if Level == 1:
             self.sosType[self.block_cntr] = cplex.Cplex.SOS.type.SOS1
-        elif level == 2:
+        elif Level == 2:
             self.sosType[self.block_cntr] = cplex.Cplex.SOS.type.SOS2
-
-        # The variable being indexed
-        var = con.sos_vars()
-        
-        if index is None:
-            tmpSet = con.sos_set()
-            self.sosName[self.block_cntr] = name
         else:
-            tmpSet = con.sos_set()[index]
-            self.sosName[self.block_cntr] = name+str(index)
+            raise ValueError("Invalid SOSConstraint level %s" % Level)
 
-        # Get all the variables
-        cntr = 1.0
-        for idx in tmpSet:
-            self.varnames[self.block_cntr].append(symbol_map.getSymbol(var[idx],labeler))
-            # We need to weight each variable
-            # For now we just increment a counter
-            self.weights[self.block_cntr].append(cntr)
-            cntr += 1.0
+        self.sosName[self.block_cntr] = symbol_map.getSymbol(soscondata,labeler)
+        
+        for var, weight in zip(Vars, Weights):
+            varids.append(id(var))
+            varnames.append(symbol_map.getSymbol(var,labeler))
+            weights.append(weight)
 
 
 class CPLEXDirect(OptSolver):
@@ -95,7 +103,6 @@ class CPLEXDirect(OptSolver):
     pyutilib.component.core.alias('_cplex_direct',  doc='Direct Python interface to the CPLEX LP/MIP solver')
 
     def __init__(self, **kwds):
-
         #
         # Call base class constructor
         #
@@ -107,7 +114,7 @@ class CPLEXDirect(OptSolver):
         # across the _presolve, _apply_solver, and _postsolve methods.
         self.warm_start_solve = False
 
-        # the working problem instance, via CPLEX python constructs.
+        # The working problem instance, via CPLEX python constructs.
         self._active_cplex_instance = None
 
         # Note: Undefined capabilities default to 'None'
@@ -119,6 +126,18 @@ class CPLEXDirect(OptSolver):
         self._capabilities.sos1 = True
         self._capabilities.sos2 = True
 
+        #
+        self.allow_callbacks = True
+
+        # the CPLEX python API doesn't provide a mechanism to track
+        # user/system/wall clock time, so it's up to us. stored as an
+        # attribute of the plugin to facilitate persistance across 
+        # various portions of the method invocations.
+        self._solve_user_time = None
+
+        # collection of id(_VarData).
+        self._referenced_variable_ids = set()
+
     def available(self, exception_flag=True):
         """ True if the solver is available """
 
@@ -158,12 +177,10 @@ class CPLEXDirect(OptSolver):
 
         from coopr.pyomo.base import expr
 
-        if isinstance(exp, expr._IdentityExpression):
-            return self._evaluate_bound(exp._args[0])
-        elif exp.is_constant() or exp.is_fixed():
+        if exp.is_fixed():
             return exp()
         else:
-            raise ValueError("ERROR: nonconstant bound: " + str(exp))
+            raise ValueError("ERROR: non-fixed bound: " + str(exp))
 
     #
     # CPLEX requires objective expressions to be specified via something other than a sparse pair!
@@ -180,7 +197,9 @@ class CPLEXDirect(OptSolver):
 
         for var_hash, var_coefficient in iteritems(expression[1]):
 
-            variable_name = self._symbol_map.getSymbol(hash_to_variable_map[var_hash], labeler)
+            vardata = hash_to_variable_map[var_hash]
+            self._referenced_variable_ids.add(id(vardata))
+            variable_name = self._symbol_map.getSymbol(vardata, labeler)
 
             if as_pairs is True:
                 pairs.append((variable_name, var_coefficient))
@@ -214,7 +233,7 @@ class CPLEXDirect(OptSolver):
 
             var_coefficient = linear_repn.linear[i]
             var_value = linear_repn.variables[i]
-
+            self._referenced_variable_ids.add(id(var_value))
             variable_name = self._symbol_map.getSymbol(var_value, labeler)
 
             if as_pairs is True:
@@ -248,13 +267,16 @@ class CPLEXDirect(OptSolver):
 
         for vrs, coeff in iteritems(expression[2]):
 
+            variable_hash_iter = iterkeys(vrs)
+            vardata = hash_to_variable_map[advance_iterator(variable_hash_iter)]
+            self._referenced_variable_ids.add(id(vardata))
+            var1 = self._symbol_map.getSymbol(vardata, labeler)
             if len(vrs)==2:
-                variable_hash_iter = iterkeys(vrs)
-                var1 = self._symbol_map.getSymbol(hash_to_variable_map[advance_iterator(variable_hash_iter)], labeler)
-                var2 = self._symbol_map.getSymbol(hash_to_variable_map[advance_iterator(variable_hash_iter)], labeler)
+                vardata = hash_to_variable_map[advance_iterator(variable_hash_iter)]
+                self._referenced_variable_ids.add(id(vardata))
+                var2 = self._symbol_map.getSymbol(vardata, labeler)
             else:
-                variable_hash_iter = iterkeys(vrs)
-                var1 = var2  = self._symbol_map.getSymbol(hash_to_variable_map[advance_iterator(variable_hash_iter)], labeler)
+                var2 = var1
 
             if as_triples is True:
                 triples.append((var1, var2, is_obj*coeff))
@@ -275,10 +297,7 @@ class CPLEXDirect(OptSolver):
     #
     def _populate_cplex_instance(self, pyomo_instance):
 
-        labeler = NumericLabeler('x')
-        self._symbol_map = SymbolMap(pyomo_instance)
-
-        from coopr.pyomo.base import Var, Objective, Constraint, ConstraintList, IntegerSet, BooleanSet, SOSConstraint
+        from coopr.pyomo.base import Var, Objective, Constraint, IntegerSet, BooleanSet, SOSConstraint
         from coopr.pyomo.base.objective import minimize, maximize
         from coopr.pyomo.expr import canonical_is_constant
         from coopr.pyomo import LinearCanonicalRepn
@@ -290,8 +309,18 @@ class CPLEXDirect(OptSolver):
         try:
             cplex_instance = cplex.Cplex()
         except CplexError:
+            e = sys.exc_info()[1]
+            msg = 'Unable to create Cplex model.  Have you installed the Python'\
+            '\n       bindings for Cplex?\n\n\tError message: %s'
             print(sys.exc_info()[1])
-            raise ValueError("TBD - FAILED TO CREATE CPLEX INSTANCE!")
+            raise Exception(msg % e)
+
+
+        if self.symbolic_solver_labels is True:
+            labeler = TextLabeler()
+        else:
+            labeler = NumericLabeler('x')
+        self._symbol_map = SymbolMap(pyomo_instance)
 
         # cplex wants the caller to set the problem type, which is (for current
         # purposes) strictly based on variable type counts.
@@ -305,9 +334,11 @@ class CPLEXDirect(OptSolver):
         var_ubs = []
         var_types = []
 
+        self._referenced_variable_ids.clear()
+
         for block in pyomo_instance.all_blocks():
             for var in active_subcomponents_data_generator(block,Var):
-                if (not var.active) or var.fixed:
+                if var.fixed is True:
                     continue
                 var_names.append(self._symbol_map.getSymbol( var, labeler ))
                 if var.lb is None:
@@ -345,7 +376,7 @@ class CPLEXDirect(OptSolver):
 
         for block in pyomo_instance.all_blocks():
 
-            block_repn_suffix = block.subcomponent("repn")
+            block_repn_suffix = block.subcomponent("canonical_repn")
             if (block_repn_suffix is None) or (not block_repn_suffix.type() is Suffix) or (not block_repn_suffix.active is True):
                 block_repn_suffix = None
 
@@ -464,25 +495,20 @@ class CPLEXDirect(OptSolver):
         sos2 = self._capabilities.sos2
         modelSOS = ModelSOS()
         for block in pyomo_instance.all_blocks():
-            for con in active_subcomponents_generator(block,SOSConstraint):
-                level = con.sos_level()
+            for soscondata in active_subcomponents_data_generator(block,SOSConstraint):
+                level = soscondata.get_level()
                 if (level == 1 and not sos1) or (level == 2 and not sos2) or (level > 2 and not sosn):
                     raise Exception("Solver does not support SOS level %s constraints" % (level,))
-                name = self._symbol_map.getSymbol( con, labeler ) 
-                masterIndex = con.sos_set_set()
-                if None in masterIndex:
-                    # A single constraint
-                    modelSOS.count_constraint(self._symbol_map,labeler,con, name, level)
-                else:
-                    # A series of indexed constraints
-                    for index in masterIndex:
-                        modelSOS.count_constraint(self._symbol_map,labeler,con, name, level, index)
+                modelSOS.count_constraint(self._symbol_map,
+                                          labeler,
+                                          soscondata)
 
         if modelSOS.sosType:
             for key in modelSOS.sosType:
                 cplex_instance.SOS.add(type = modelSOS.sosType[key], \
                                        name = modelSOS.sosName[key], \
                                        SOS = [modelSOS.varnames[key], modelSOS.weights[key]] )
+                self._referenced_variable_ids.update(modelSOS.varids[key])
             used_sos_constraints = True
 
         cplex_instance.linear_constraints.add(lin_expr=expressions, senses=senses, rhs=rhss, range_values=range_values, names=names)
@@ -490,9 +516,9 @@ class CPLEXDirect(OptSolver):
         for index in xrange(len(qexpressions)):
             cplex_instance.quadratic_constraints.add(lin_expr=qlinears[index], quad_expr=qexpressions[index], sense=qsenses[index], rhs=qrhss[index], name=qnames[index])
         
-        model_repn_suffix = pyomo_instance.subcomponent("repn")
+        model_repn_suffix = pyomo_instance.subcomponent("canonical_repn")
         if (model_repn_suffix is None) or (not model_repn_suffix.type() is Suffix) or (not model_repn_suffix.active is True):
-            raise ValueError("Unable to find an active Suffix with name 'repn' on block: %s" % (pyomo_instance.cname(True)))
+            raise ValueError("Unable to find an active Suffix with name 'canonical_repn' on block: %s" % (pyomo_instance.cname(True)))
 
         # transfer the objective.
         for obj_data in active_subcomponents_data_generator(pyomo_instance,Objective):
@@ -578,31 +604,67 @@ class CPLEXDirect(OptSolver):
 
         from coopr.pyomo.base import Var
 
-        labeler = NumericLabeler()
         # the iteration order is identical to that used in generating
         # the cplex instance, so all should be well.
         variable_names = []
         variable_values = []
-        for var in active_subcomponents_data_generator(Var):
-            if (var.stale is True) or (var.fixed is True):
-                continue
-            name = self._symbol_map.getSymbol(var, _error_labeler)
-            variable_names.append(name)
-            variable_values.append(var.value)
+        for block in instance.all_blocks():
+            for var in active_subcomponents_data_generator(block, Var):
+                if (var.stale is True) or (var.fixed is True):
+                    continue
+                name = self._symbol_map.getSymbol(var, _error_labeler)
+                variable_names.append(name)
+                variable_values.append(var.value)
 
         self._active_cplex_instance.MIP_starts.add([variable_names, variable_values],
                                                    self._active_cplex_instance.MIP_starts.effort_level.auto)
 
-
     # over-ride presolve to extract the warm-start keyword, if specified.
     def _presolve(self, *args, **kwds):
         from coopr.pyomo.base.PyomoModel import Model
 
-        self.suffixes = kwds.pop( 'suffixes', self.suffixes )
-        self.tee = kwds.pop( 'tee', False )
-        self.keepfiles = kwds.pop( 'keepfiles', False )
-        self.warm_start_solve = kwds.pop( 'warmstart', False )
+        self.warm_start_solve = False
+        self.keepfiles = False
+        self.tee = False
+        self.symbolic_solver_labels = False
+        for key in kwds:
+            ### copied from base class _presolve
+            warn = False
+            if key == "logfile":
+                if kwds[key] is not None:
+                    warn = True
+            elif key == "solnfile":
+                if kwds[key] is not None:
+                    warn = True
+            elif key == "timelimit":
+                if kwds[key] is not None:
+                    warn = True
+            elif key == "tee":
+                self.tee=bool(kwds[key])
+            elif key == "options":
+                self.set_options(kwds[key])
+            elif key == "available":
+                self._assert_available=True
+            elif key == "symbolic_solver_labels":
+                self.symbolic_solver_labels = bool(kwds[key])
+            elif key == "output_fixed_variable_bounds":
+                if kwds[key] is not False:
+                    warn = True
+            elif key == "suffixes":
+                self.suffixes=kwds[key]
+            ###
+            elif key == 'keepfiles':
+                self.keepfiles = bool(kwds[key])
+            elif key == 'warmstart':
+                self.warm_start_solve = bool(kwds[key])
+            else:
+                raise ValueError("Unknown option="+key+" for solver="+self.type)
 
+            if warn is True:
+                logger.warn('"'+key+'" keyword ignored by solver='+self.type)
+
+        self.available()
+        
         # Step 1: extract the pyomo instance from the input arguments,
         #         cache it, and create the corresponding (as of now empty)
         #         CPLEX problem instance.
@@ -688,14 +750,22 @@ class CPLEXDirect(OptSolver):
             #self._active_cplex_instance.set_log_stream(log_file)
         else:
             self._active_cplex_instance.set_results_stream(None)
-        self._active_cplex_instance.solve()
+
+        # apparently some versions of the CPLEX Python bindings do not
+        # have the get_time - so check before accessing.
+        if hasattr(self._active_cplex_instance, "get_time"):
+            solve_start_time = self._active_cplex_instance.get_time()
+            self._active_cplex_instance.solve()
+            self._solve_user_time = self._active_cplex_instance.get_time() - solve_start_time
+        else:
+            self._active_cplex_instance.solve()
+            self._solve_user_time = None
 
         # FIXME: can we get a return code indicating if CPLEX had a
         # significant failure?
         return Bunch(rc=None, log=None)
 
     def _postsolve(self):
-
         # the only suffixes that we extract from CPLEX are
         # constraint duals, constraint slacks, and variable
         # reduced-costs. scan through the solver suffix list
@@ -718,10 +788,8 @@ class CPLEXDirect(OptSolver):
             if not flag:
                 raise RuntimeError("***The CPLEXDirect solver plugin cannot extract solution suffix="+suffix)
 
-
         instance = self._active_cplex_instance
 
-
         if instance.get_problem_type() in [instance.problem_type.MILP,
                                            instance.problem_type.MIQP,
                                            instance.problem_type.MIQCP]:
@@ -755,12 +823,14 @@ class CPLEXDirect(OptSolver):
 #        results.solver.status = None
         results.solver.return_code = None
         results.solver.message = None
-        results.solver.user_time = None
+        results.solver.user_time = self._solve_user_time
         results.solver.system_time = None
         results.solver.wallclock_time = None
         results.solver.termination_message = None
         
         soln = Solution()
+        soln_variable = soln.variable
+        soln_constraint = soln.constraint
 
         #Get solution status -- for now, if CPLEX returns anything we don't recognize, mark as an error
         soln_status = instance.solution.get_status()
@@ -787,35 +857,41 @@ class CPLEXDirect(OptSolver):
             variable_values = instance.solution.get_values()
             for i in xrange(num_variables):
                 variable_name = variable_names[i]
-                soln.variable[variable_name] = {"Value" : variable_values[i], "Id" : len(soln.variable)}
-
+                soln_variable[variable_name] = {"Value" : variable_values[i]}
+            
             if extract_reduced_costs:
                 # get variable reduced costs
                 rc_values = instance.solution.get_reduced_costs()
                 for i in xrange(num_variables):
-                    soln.variable[variable_names[i]]["Rc"] = rc_values[i]
+                    soln_variable[variable_names[i]]["Rc"] = rc_values[i]
 
+            num_linear_constraints = instance.linear_constraints.get_num()
+            constraint_names = instance.linear_constraints.get_names()
+            
+            num_quadratic_constraints = instance.quadratic_constraints.get_num()
+            q_constraint_names = instance.quadratic_constraints.get_names()
+
+            if extract_slacks or extract_duals:
+                for i in xrange(num_linear_constraints):
+                    soln_constraint[constraint_names[i]] = {}
+                    
             if extract_duals:
                 # get duals (linear constraints only)
-                num_linear_constraints = instance.linear_constraints.get_num()
-                constraint_names = instance.linear_constraints.get_names()
-                dual_values = instance.solution.get_dual_values() 
+                dual_values = instance.solution.get_dual_values()
                 for i in xrange(num_linear_constraints):
-                    soln.constraint[constraint_names[i]].dual = dual_values[i]
+                    soln_constraint[constraint_names[i]]["Dual"] = dual_values[i]
 
                 # CPLEX PYTHON API DOES NOT SUPPORT QUADRATIC DUAL COLLECTION
 
             if extract_slacks:
                 # get linear slacks
-                num_linear_constraints = instance.linear_constraints.get_num()
-                constraint_names = instance.linear_constraints.get_names()
                 slack_values = instance.solution.get_linear_slacks() 
                 for i in xrange(num_linear_constraints):
                     # if both U and L exist (i.e., a range constraint) then 
                     # R_ = U-L
                     R_ = instance.linear_constraints.get_range_values(i)
                     if R_ == 0.0:
-                        soln.constraint[constraint_names[i]].slack = slack_values[i]
+                        soln_constraint[constraint_names[i]]["Slack"] = slack_values[i]
                     else:
                         # This is a range constraint for which cplex always returns the
                         # value of f(x)-L. In the spirit of conforming with the other writer,
@@ -823,18 +899,22 @@ class CPLEXDirect(OptSolver):
                         Ls_ = slack_values[i]
                         Us_ = R_ - slack_values[i]
                         if Us_ > Ls_:
-                            soln.constraint[constraint_names[i]].slack = Us_
+                            soln_constraint[constraint_names[i]]["Slack"] = Us_
                         else:
-                            soln.constraint[constraint_names[i]].slack = -Ls_
+                            soln_constraint[constraint_names[i]]["Slack"] = -Ls_
                 
                 # get quadratic slacks
-                num_quadratic_constraints = instance.quadratic_constraints.get_num()
-                constraint_names = instance.quadratic_constraints.get_names()
                 slack_values = instance.solution.get_quadratic_slacks() 
                 for i in xrange(num_quadratic_constraints):
                     # if both U and L exist (i.e., a range constraint) then 
                     # R_ = U-L
-                    soln.constraint[constraint_names[i]].slack = slack_values[i]
+                    soln_constraint[q_constraint_names[i]] = {"Slack" : slack_values[i]}
+            
+            byObject = self._symbol_map.byObject
+            referenced_varnames = set(byObject[varid] for varid in self._referenced_variable_ids)
+            names_to_delete = set(soln_variable.keys())-referenced_varnames
+            for varname in names_to_delete:
+                del soln_variable[varname]
 
             results.solution.insert(soln)
 
@@ -869,6 +949,41 @@ class CPLEXDirect(OptSolver):
         # let the base class deal with returning results.
         return OptSolver._postsolve(self)
 
+    def _initialize_callbacks(self, model):
+        #
+        # Called from OptSolver
+        #
+        cplex_callback = {
+            "node-callback":        cplex.callbacks.NodeCallback,
+            "solve-callback":       cplex.callbacks.SolveCallback,
+            "branch-callback":      cplex.callbacks.BranchCallback,
+            "heuristic-callback":   cplex.callbacks.HeuristicCallback,
+            "incumbent-callback":   cplex.callbacks.IncumbentCallback,
+            "cut-callback":         cplex.callbacks.UserCutCallback,
+            "lazycut-callback":     cplex.callbacks.LazyConstraintCallback,
+            "crossover-callback":   cplex.callbacks.CrossoverCallback,
+            "barrier-callback":     cplex.callbacks.BarrierCallback,
+            "simplex-callback":     cplex.callbacks.SimplexCallback,
+            "presolve-callback":    cplex.callbacks.PresolveCallback,
+            "tuning-callback":      cplex.callbacks.TuningCallback
+            }
+        #
+        for name in self._callback:
+            try:
+                cb_class = cplex_callback[name]
+            except KeyError:
+                raise ValueError("Unknown callback name: %s" % name)
+            #
+            def call_fn(self, *args, **kwds):
+                try:
+                    self.solver = CplexSolverWrapper(self)
+                    self._callback[self.name](self.solver, model)
+                except Exception(e):
+                    # Should we raise this exception?
+                    print("ERROR: "+str(e))
+            CallbackClass = type('CallbackClass_'+name.replace('-','_'), (cb_class,object), {"_callback":self._callback, "name":name, "__call__":call_fn})
+            self._active_cplex_instance.register_callback(CallbackClass)
+
 
 if cplex_import_available is False:
     SolverFactory().deactivate('_cplex_direct')
