@@ -205,7 +205,7 @@ class GLPKSHELL ( SystemCallSolver ):
         """
         results = SolverResults()
 
-          # For the lazy programmer, handle long variable names
+        # For the lazy programmer, handle long variable names
         prob   = results.problem
         solv   = results.solver
         solv.termination_condition = TerminationCondition.unknown
@@ -298,17 +298,12 @@ class GLPKSHELL ( SystemCallSolver ):
             extract_duals = False
             extract_reduced_costs = False
             for suffix in self.suffixes:
-                flag = False
                 if re.match(suffix, "dual"):
-                    flag = True
                     if not self.is_integer:
                         extract_duals = True
-                if re.match(suffix, "rc"):
-                    flag = True
+                elif re.match(suffix, "rc"):
                     if not self.is_integer:
                         extract_reduced_costs = True
-                if not flag:
-                    raise RuntimeError("***The GLPK solver plugin cannot extract solution suffix="+suffix)
 
             for line in f:
                 glp_line_count += 1
@@ -397,22 +392,24 @@ class GLPKSHELL ( SystemCallSolver ):
                 # less 'arbitrary', as in the yaml key 'f'.  Weird
                 soln.objective[ obj_name ] = obj_val
 
-                for mm in range( 1, prows +1 ):
-                    raw_line_count += 1
-                    if self.is_integer:
-                        rprim = f.readline()   # should be a single number
-                    else:
+                if (self.is_integer is True) or (extract_duals is False):
+                    # we use nothing from this section so just read in the
+                    # lines and throw them away
+                    for mm in range( 1, prows +1 ):
+                        raw_line_count += 1
+                        f.readline()
+                else:
+                    for mm in range( 1, prows +1 ):
+                        raw_line_count += 1                    
+
                         rstat, rprim, rdual = f.readline().split()
                         rstat = float( rstat )
+                        
+                        cname = constraint_names[ mm ]
+                        if 'ONE_VAR_CONSTANT' == cname[-16:]: continue
 
-                    cname = constraint_names[ mm ]
-                    if 'ONE_VAR_CONSTANT' == cname[-16:]: continue
-                    rprim = float( rprim )
-                    soln.constraint[ cname ].value = rprim
-
-                    if extract_duals:
                         if cname.startswith('c_'):
-                            soln.constraint[ cname ].dual  = float(rdual)
+                            soln.constraint[ cname ] = {"Dual":float(rdual)}
                         elif cname.startswith('r_l_'):
                             range_duals.setdefault(cname[4:],[0,0])[0] = float(rdual)
                         elif cname.startswith('r_u_'):
@@ -444,13 +441,13 @@ class GLPKSHELL ( SystemCallSolver ):
 
         if not soln is None:
             # For the range constraints, supply only the dual with the largest
-            # magnitude (at least one should always be numericly zero)
+            # magnitude (at least one should always be numerically zero)
             scon = soln.Constraint
             for key,(ld,ud) in iteritems(range_duals):
                 if abs(ld) > abs(ud):
-                    scon['r_l_'+key].dual = ld
+                    scon['r_l_'+key] = {"Dual":ld}
                 else:
-                    scon['r_u_'+key].dual = ud
+                    scon['r_u_'+key] = {"Dual":ud}
 
 
 #class MockGLPK(GLPK,mockmip.MockMIP):
