@@ -316,7 +316,7 @@ class GLPKSHELL_old(SystemCallSolver):
                         elif (len(tokens) == 2) and (tokens[1] == "UNDEFINED"):
                             soln.status = SolutionStatus.infeasible
                         else:
-                            print ("WARNING: Read unknown status while " + \
+                            print("WARNING: Read unknown status while " + \
                                    "parsing GLPK solution file - " + \
                                    "status='%s'") % (" ".join(tokens[1:]))
                     elif len(tokens) >= 2 and tokens[0] == "Objective:":
@@ -416,7 +416,7 @@ class GLPKSHELL_old(SystemCallSolver):
                             if index != number_of_constraints_read:
                                 raise ValueError("***ERROR: Unexpected constraint index encountered on line="+line+"; expected value="+str(number_of_constraints_read)+"; actual value="+str(index))
 
-                        if lp_solution is True:
+                        if (lp_solution is True) and (extract_duals is True):
                             # GLPK doesn't report slacks directly.
                             constraint_dual = activity
                             if state_string == "B":
@@ -426,13 +426,12 @@ class GLPKSHELL_old(SystemCallSolver):
                             else:
                                 raise ValueError("Unknown status="+tokens[0]+" encountered for constraint="+active_constraint_name+" in line="+line+" of solution file="+self.soln_file)
 
-                            if extract_duals is True:
-                                if constraint_name.startswith('c_'):
-                                    soln.constraint[constraint_name].dual  = float(constraint_dual)
-                                elif constraint_name.startswith('r_l_'):
-                                    range_duals.setdefault(constraint_name[4:],[0,0])[0] = float(constraint_dual)
-                                elif constraint_name.startswith('r_u_'):
-                                    range_duals.setdefault(constraint_name[4:],[0,0])[1] = float(constraint_dual)
+                            if constraint_name.startswith('c_'):
+                                soln.constraint[constraint_name] = {"Dual" : float(constraint_dual)}
+                            elif constraint_name.startswith('r_l_'):
+                                range_duals.setdefault(constraint_name[4:],[0,0])[0] = float(constraint_dual)
+                            elif constraint_name.startswith('r_u_'):
+                                range_duals.setdefault(constraint_name[4:],[0,0])[1] = float(constraint_dual)
 
                         else:
                             # there isn't anything interesting to do with constraints in the MIP case.
@@ -567,13 +566,13 @@ class GLPKSHELL_old(SystemCallSolver):
             raise
 
         # For the range constraints, supply only the dual with the largest
-        # magnitude (at least one should always be numericly zero)
+        # magnitude (at least one should always be numerically zero)
         scon = soln.Constraint
         for key,(ld,ud) in range_duals.items():
             if abs(ld) > abs(ud):
-                scon['r_l_'+key].dual = ld
+                scon['r_l_'+key] = {"Dual" : ld, "Id" : len(scon)}
             else:
-                scon['r_u_'+key].dual = ud
+                scon['r_u_'+key] = {"Dual" : ud, "Id" : len(scon)}
 
         #
         if soln.status is SolutionStatus.optimal:
