@@ -62,7 +62,7 @@ class GUROBI(OptSolver):
         if mode == 'os':
             opt = SolverFactory('_ossolver', **kwds)
         elif mode == 'nl':
-            opt = SolverFactory('_asl', **kwds)
+            opt = SolverFactory('asl', **kwds)
         else:
             logging.getLogger('coopr.plugins').error('Unknown IO type: %s' % mode)
             return
@@ -218,7 +218,7 @@ class GUROBISHELL(ILMLicensedSystemCallSolver):
         outname = pyutilib.services.TempfileManager.create_tempfile(suffix = '.gurobi.version')
         with open(outname,'w') as f:
             # **Note, adding a 'timelimit' keyword here results in empty output for some reason
-            results = pyutilib.subprocess.run( [solver_exec], stdin='from gurobipy import *; print gurobi.version(); exit()', ostream=f) 
+            results = pyutilib.subprocess.run( [solver_exec], stdin='from gurobipy import *; print(gurobi.version()); exit()', ostream=f) 
         tmp = None
         with open(outname,'r') as f:
             tmp = tuple(eval(f.read().strip()))
@@ -299,6 +299,29 @@ class GUROBISHELL(ILMLicensedSystemCallSolver):
 
     def process_soln_file(self,results):
 
+
+        # the only suffixes that we extract from CPLEX are
+        # constraint duals, constraint slacks, and variable
+        # reduced-costs. scan through the solver suffix list
+        # and throw an exception if the user has specified
+        # any others.
+        extract_duals = False
+        extract_slacks = False
+        extract_rc = False
+        for suffix in self.suffixes:
+            flag=False
+            if re.match(suffix,"dual"):
+                extract_duals = True
+                flag=True
+            if re.match(suffix,"slack"):
+                extract_slacks = True
+                flag=True
+            if re.match(suffix,"rc"):
+                extract_rc = True
+                flag=True
+            if not flag:
+                raise RuntimeError("***The GUROBI solver plugin cannot extract solution suffix="+suffix)
+
         # check for existence of the solution file
         # not sure why we just return - would think that we
         # would want to indicate some sort of error
@@ -307,10 +330,12 @@ class GUROBISHELL(ILMLicensedSystemCallSolver):
 
         soln = Solution()
 
+        # caching for efficiency
         soln_variables = soln.variable
-        num_variables_read = 0
         soln_constraints = soln.constraint
 
+        num_variables_read = 0
+
         # string compares are too expensive, so simply introduce some section IDs.
         # 0 - unknown
         # 1 - problem
@@ -323,6 +348,7 @@ class GUROBISHELL(ILMLicensedSystemCallSolver):
         
         range_duals = {}
         range_slacks = {}
+
         INPUT = open(self.soln_file,"r")
         for line in INPUT:
             line = line.strip()
@@ -355,7 +381,7 @@ class GUROBISHELL(ILMLicensedSystemCallSolver):
                         name = tokens[1]
                         if name != "c_e_ONE_VAR_CONSTANT":
                             if name.startswith('c_'):
-                                soln_constraints[tokens[1]].dual = float(tokens[2])
+                                soln_constraints.setdefault(tokens[1],{"Id" : len(soln_constraints)})["Dual"] = float(tokens[2])
                             elif name.startswith('r_l_'):
                                 range_duals.setdefault(name[4:],[0,0])[0] = float(tokens[2])
                             elif name.startswith('r_u_'):
@@ -364,7 +390,7 @@ class GUROBISHELL(ILMLicensedSystemCallSolver):
                         name = tokens[1]
                         if name != "c_e_ONE_VAR_CONSTANT":
                             if name.startswith('c_'):
-                                soln_constraints[tokens[1]].slack = float(tokens[2])
+                                soln_constraints.setdefault(tokens[1],{"Id" : len(soln_constraints)})["Slack"] = float(tokens[2])
                             elif name.startswith('r_l_'):
                                 range_slacks.setdefault(name[4:],[0,0])[0] = float(tokens[2])
                             elif name.startswith('r_u_'):
@@ -387,37 +413,31 @@ class GUROBISHELL(ILMLicensedSystemCallSolver):
                             val = tokens[1]
                         setattr(results.problem, tokens[0], val)
                 elif (section == 3):
-                    try:
-                        val = eval(tokens[1])
-                    except:
-                        val = tokens[1]
                     if (tokens[0] == 'status'):
-                        results.solver.status = getattr(SolverStatus, val)
+                        results.solver.status = getattr(SolverStatus, tokens[1])
                     elif (tokens[0] == 'termination_condition'):
                         try:
-                            results.solver.termination_condition = getattr(TerminationCondition, val)
+                            results.solver.termination_condition = getattr(TerminationCondition, tokens[1])
                         except AttributeError:
                             results.solver.termination_condition = TerminationCondition.unknown
                     else:
-                        setattr(results.solver, tokens[0], val)
+                        setattr(results.solver, tokens[0], tokens[1])
 
         INPUT.close()
-
+        
         # For the range constraints, supply only the dual with the largest
-        # magnitude (at least one should always be numericly zero)
-        scon = soln.Constraint
+        # magnitude (at least one should always be numerically zero)
         for key,(ld,ud) in iteritems(range_duals):
             if abs(ld) > abs(ud):
-                scon['r_l_'+key].dual = ld
+                soln_constraints['r_l_'+key] = {"Dual" : ld, "Id" : len(soln_constraints)}
             else:
-                scon['r_u_'+key].dual = ud
+                soln_constraints['r_u_'+key] = {"Dual" : ud, "Id" : len(soln_constraints)}
         # slacks
         for key,(ls,us) in iteritems(range_slacks):
             if abs(ls) > abs(us):
-                scon['r_l_'+key].slack = ls
+                soln_constraints.setdefault('r_l_'+key,{"Id" : len(soln_constraints)})["Slack"] = ls
             else:
-                scon['r_u_'+key].slack = us
-
+                soln_constraints.setdefault('r_u_'+key,{"Id" : len(soln_constraints)})["Slack"] = us
 
         if solution_seen is True:
             results.solution.insert(soln)
