@@ -206,6 +206,10 @@ class PICOSHELL(SystemCallSolver):
         elif self._problem_format == ProblemFormat.cpxlp \
                 or self._problem_format == ProblemFormat.mps:
             cmd.append('--allowInfiniteIntegerVarBounds=true')
+            # This option should appear in the next PICO release,
+            # assuming that ever happens
+            if self.version() > (1,3,1,0):
+                cmd.append('--reportDenseSolution=true')
             if "debug" not in self.options:
                 cmd.extend(["--debug", "2"])
             for key, val in _check_and_escape_options(self.options):
@@ -370,12 +374,23 @@ class PICOSHELL(SystemCallSolver):
                     continue
                 soln.variable[var] = {"Value" : val, "Id" : len(soln.variable)}
         else:
+            range_duals = {}
+            soln_constraints = soln.constraint
             if (lp_flag is True) and (extract_duals is True):
                 for (var,val) in tmp:
-                    soln.constraint[var].dual = val
-            else:
-                for (var,val) in tmp:
-                    soln.constraint[var] = val
+                    if var.startswith('c_'):
+                        soln_constraints[var] = {"Dual" : val, "Id" : len(soln_constraints)}
+                    elif var.startswith('r_l_'):
+                        range_duals.setdefault(var[4:],[0,0])[0] = val
+                    elif var.startswith('r_u_'):
+                        range_duals.setdefault(var[4:],[0,0])[1] = val
+            # For the range constraints, supply only the dual with the largest
+            # magnitude (at least one should always be numerically zero)
+            for key,(ld,ud) in iteritems(range_duals):
+                if abs(ld) > abs(ud):
+                    soln_constraints['r_l_'+key] = {"Dual" : ld, "Id" : len(soln_constraints)}
+                else:
+                    soln_constraints['r_u_'+key] = {"Dual" : ud, "Id" : len(soln_constraints)}
         INPUT.close()
 
 
