@@ -376,12 +376,12 @@ class XPRESS_shell(ILMLicensedSystemCallSolver):
                         
             else:
                 
-                constraint = soln.constraint[name]
+                constraint = soln.constraint[name] = {"Id" : len(soln.constraint)}
                 
                 if (extract_duals is True) and (tertiary_value != 0.0):
-                    constraint.dual = tertiary_value
+                    constraint["Dual"] = tertiary_value
                 if (extract_slacks is True) and (secondary_value != 0.0):
-                    constraint.slack = secondary_value
+                    constraint["Slack"] = secondary_value
 
         if not results.solver.status is SolverStatus.error and \
             results.solver.termination_condition in [TerminationCondition.unknown,
