@@ -8,7 +8,7 @@
 #  the U.S. Government retains certain rights in this software.
 #  For more information, see the Coopr README.txt file.
 #  _________________________________________________________________________
-
+import sys
 import logging
 _glpk_version = None
 try:
@@ -103,7 +103,6 @@ class GLPKDirect ( OptSolver ):
     def _populate_glpk_instance ( self, model ):
         from coopr.pyomo.base import Var, Objective, Constraint, \
                                      IntegerSet, BooleanSet, SOSConstraint
-        from coopr.pyomo.expr import is_constant
 
         try:
             lp = glp_create_prob()
@@ -128,12 +127,10 @@ class GLPKDirect ( OptSolver ):
         sos1 = self._capabilities.sos1
         sos2 = self._capabilities.sos2
 
-        sos_con_list = model.active_components(SOSConstraint)
-        if not(sos1 or sos2 or sosn):
-            if len(sos_con_list) > 0:
+        for block in pyomo_instance.all_blocks():
+            for soscondata in active_subcomponents_data_generator(block,SOSConstraint):
                 raise Exception("Solver: glpk_direct does not support SOSConstraint declarations")
         
-
         glp_set_prob_name( lp, model.name )
 
         glp_set_obj_dir( lp, sense )
@@ -155,7 +152,7 @@ class GLPKDirect ( OptSolver ):
             var_set = variable_list[ name ]
             for ii in var_set.keys():
                 var = var_set[ ii ]
-                if (not var.active) or (var.fixed is True):
+                if var.fixed is True:
                     continue
 
                 lb = ub = 0.0
@@ -187,9 +184,9 @@ class GLPKDirect ( OptSolver ):
                 else:
                     glp_set_col_kind( lp, col, GLP_CV )   # continuous
 
-        model_repn_suffix = model.subcomponent("repn")
+        model_repn_suffix = model.subcomponent("canonical_repn")
         if (model_repn_suffix is None) or (not model_repn_suffix.type() is Suffix) or (not model_repn_suffix.active is True):
-            raise ValueError("Unable to find an active Suffix with name 'repn' on block: %s" % (model.cname(True)))
+            raise ValueError("Unable to find an active Suffix with name 'canonical_repn' on block: %s" % (model.cname(True)))
         for name in constraint_list.keys():
             constraint_set = constraint_list[ name ]
             if constraint_set.trivial: continue
@@ -246,7 +243,7 @@ class GLPKDirect ( OptSolver ):
 
         for key in objective:
             expression = model_repn_suffix.getValue(objective[ key ])
-            if is_constant( expression ):
+            if expression.is_constant():
                 msg = "Ignoring objective '%s[%s]' which is constant"
                 logger.warning( msg % (str(objective), str(key)) )
                 continue
@@ -556,7 +553,7 @@ class GLPKDirect ( OptSolver ):
 
             for row_label in rowvar_map.keys():
                 row = rowvar_map[ row_label ]
-                soln.constraint[ row_label ] = get_row_prim( lp, row )
+                soln.constraint[ row_label ] = {"Value" : get_row_prim( lp, row ), "Id" : len(soln.constraint)}
 
         results.solution.insert(soln)
 
