@@ -11,6 +11,7 @@
 import logging
 import re
 import itertools
+import math
 
 from six import itervalues, iteritems, advance_iterator
 
@@ -27,6 +28,15 @@ try:
     gurobi_python_api_exists = True
 except ImportError:
     gurobi_python_api_exists = False
+except Exception as e: 
+    # other forms of exceptions can be thrown by the gurobi python
+    # import. for example, a gurobipy.GurobiError exception is thrown 
+    # if all tokens for Gurobi are already in use. assuming, of 
+    # course, the license is a token license. unfortunately, you 
+    # can't import without a license, which means we can't test 
+    # for the exception above!
+    print("Import of gurobipy failed - gurobi message="+str(e)+"\n")
+    gurobi_python_api_exists = False
 
 from pyutilib.misc import Bunch, Options
 from pyutilib.component.core import alias
@@ -36,7 +46,7 @@ from coopr.opt.base import *
 from coopr.opt.base.solvers import _extract_version
 from coopr.opt.results import *
 from coopr.opt.solver import *
-from coopr.pyomo.base import SymbolMap, NumericLabeler, Suffix
+from coopr.pyomo.base import SymbolMap, NumericLabeler, Suffix, TextLabeler
 from coopr.pyomo.base.numvalue import value
 from coopr.pyomo.base.block import active_subcomponents_generator, active_subcomponents_data_generator
 
@@ -50,37 +60,31 @@ class ModelSOS(object):
         self.sosType = {}
         self.sosName = {}
         self.varnames = {}
+        self.varids = {}
         self.weights = {}
         self.block_cntr = 0
 
-    def count_constraint(self,symbol_map,labeler,gurobi_var_map,con,name,level,index=None):
+    def count_constraint(self,symbol_map,labeler,gurobi_var_map,soscondata):
+
+        Vars = soscondata.get_members()
+        Weights = soscondata.get_weights()
+        Level = soscondata.get_level()
 
         self.block_cntr += 1
-        self.varnames[self.block_cntr] = []
-        self.weights[self.block_cntr] = []
-        if level == 1:
+        varnames = self.varnames[self.block_cntr] = []
+        varids = self.varids[self.block_cntr] = []
+        weights = self.weights[self.block_cntr] = []
+        if Level == 1:
             self.sosType[self.block_cntr] = GRB.SOS_TYPE1
-        elif level == 2:
+        elif Level == 2:
             self.sosType[self.block_cntr] = GRB.SOS_TYPE2
 
-        # The variable being indexed
-        var = con.sos_vars()
-
-        if index is None:
-            tmpSet = con.sos_set()
-            self.sosName[self.block_cntr] = name
-        else:
-            tmpSet = con.sos_set()[index]
-            self.sosName[self.block_cntr] = name+str(index)
-
-        # Get all the variables
-        cntr = 1.0
-        for idx in tmpSet:
-            self.varnames[self.block_cntr].append(gurobi_var_map[symbol_map.getSymbol(var[idx],labeler)])
-            # We need to weight each variable
-            # For now we just increment a counter
-            self.weights[self.block_cntr].append(cntr)
-            cntr += 1.0
+        self.sosName[self.block_cntr] = symbol_map.getSymbol(soscondata,labeler)
+        
+        for var, weight in zip(Vars, Weights):
+            varids.append(id(var))
+            varnames.append(gurobi_var_map[symbol_map.getSymbol(var,labeler)])
+            weights.append(weight)
 
 class gurobi_direct ( OptSolver ):
     """The Gurobi optimization solver (direct API plugin)
@@ -124,6 +128,10 @@ class gurobi_direct ( OptSolver ):
         OptSolver.__init__(self, **kwds)
 
         self._model = None
+        
+        # a dictionary that maps pyomo _VarData labels to the corresponding Gurobi 
+        # variable object. created each time _populate_gurobi_instance is called.
+        self._pyomo_gurobi_variable_map = None
 
         # NOTE: eventually both of the following attributes should be migrated
         # to a common base class.  Is the current solve warm-started?  A
@@ -140,6 +148,9 @@ class gurobi_direct ( OptSolver ):
         self._capabilities.sos1 = True
         self._capabilities.sos2 = True
 
+        # collection of id(_VarData).
+        self._referenced_variable_ids = set()
+
     def version(self):
         if _gurobi_version is None:
             return _extract_version('')
@@ -156,39 +167,51 @@ class gurobi_direct ( OptSolver ):
             else:
                 return True        
 
-    def _populate_gurobi_instance ( self, model ):
+    def _populate_gurobi_instance ( self, pyomo_instance ):
 
         from coopr.pyomo.base import Var, Objective, Constraint, ConstraintList, IntegerSet, BooleanSet, SOSConstraint
         from coopr.pyomo.expr import canonical_is_constant
         from coopr.pyomo import LinearCanonicalRepn
 
         try:
-            grbmodel = Model( name=model.name )
+            grbmodel = Model(name=pyomo_instance.name)
         except Exception:
             e = sys.exc_info()[1]
             msg = 'Unable to create Gurobi model.  Have you installed the Python'\
             '\n       bindings for Gurobi?\n\n\tError message: %s'
             raise Exception(msg % e)
 
-        labeler = NumericLabeler('x')
-        self._symbol_map = SymbolMap(model)
+        if self.symbolic_solver_labels is True:
+            labeler = TextLabeler()
+        else:
+            labeler = NumericLabeler('x')
+        self._symbol_map = SymbolMap(pyomo_instance)
 
-        pyomo_gurobi_variable_map = {} # maps labels to the corresponding Gurobi variable object
-        
-        for block in model.all_blocks():
-            for var_value in active_subcomponents_data_generator(block,Var):
-                if (not var_value.active) or (var_value.fixed is True):
+        # cache to avoid dictionary getitem calls in the loops below.
+        self_symbol_map = self._symbol_map
+
+        pyomo_gurobi_variable_map = {} # maps _VarData labels to the corresponding Gurobi variable object
+
+        self._referenced_variable_ids.clear()
+
+        # cache to avoid dictionary getitem calls in the loop below.
+        grb_infinity = GRB.INFINITY
+
+        for block in pyomo_instance.all_blocks():
+            for var_value in active_subcomponents_data_generator(block, Var):
+                if var_value.fixed is True:
                     continue
     
-                lb = -GRB.INFINITY
-                ub = GRB.INFINITY
+                lb = -grb_infinity
+                ub = grb_infinity
     
                 if var_value.lb is not None:
                     lb = value(var_value.lb)
                 if var_value.ub is not None:
                     ub = value(var_value.ub)
     
-                var_value_label = self._symbol_map.getSymbol(var_value, labeler)
+                # _VarValue objects will not be in the symbol map yet, so avoid some checks.
+                var_value_label = self_symbol_map.createSymbol(var_value, labeler)
                 
                 # be sure to impart the integer and binary nature of any variables
                 if isinstance(var_value.domain, IntegerSet):
@@ -205,22 +228,17 @@ class gurobi_direct ( OptSolver ):
 
         grbmodel.update() 
 
-        model_repn_suffix = model.subcomponent("repn")
-        if (model_repn_suffix is None) or (not model_repn_suffix.type() is Suffix) or (not model_repn_suffix.active is True):
-            raise ValueError("Unable to find an active Suffix with name 'repn' on block: %s" % (model.cname(True)))
+        pyomo_instance_repn_suffix = pyomo_instance.subcomponent("canonical_repn")
+        if (pyomo_instance_repn_suffix is None) or (not pyomo_instance_repn_suffix.type() is Suffix) or (not pyomo_instance_repn_suffix.active is True):
+            raise ValueError("Unable to find an active Suffix with name 'canonical_repn' on block: %s" % (pyomo_instance.cname(True)))
 
-        for obj_data in active_subcomponents_data_generator(model,Objective):
+        for obj_data in active_subcomponents_data_generator(pyomo_instance, Objective):
             sense = GRB_MAX
             if obj_data.is_minimizing(): sense = GRB_MIN
             grbmodel.ModelSense = sense
             obj_expr = LinExpr()
 
-            obj_repn = model_repn_suffix.getValue(obj_data)
-
-            if (isinstance(obj_repn, LinearCanonicalRepn) and (obj_repn.linear == None)) or canonical_is_constant(obj_repn):
-                msg = "Ignoring objective '%s[%s]' which is constant"
-                logger.warning( msg % (str(objective), str(key)) )
-                continue
+            obj_repn = pyomo_instance_repn_suffix.getValue(obj_data)
 
             if isinstance(obj_repn, LinearCanonicalRepn):
 
@@ -231,13 +249,10 @@ class gurobi_direct ( OptSolver ):
 
                     for i in xrange(len(obj_repn.linear)):
                         var_coefficient = obj_repn.linear[i]
-                        var_value = obj_repn.variables[i]                    
-
-                        label = self._symbol_map.getSymbol(var_value, labeler)
+                        var_value = obj_repn.variables[i]
+                        self._referenced_variable_ids.add(id(var_value))
+                        label = self_symbol_map.getSymbol(var_value, labeler)
                         obj_expr.addTerms(var_coefficient, pyomo_gurobi_variable_map[label])
-                        # the coefficients are attached to the model when creating the
-                        # variables, below
-
             else:
 
                 if 0 in obj_repn: # constant term
@@ -246,10 +261,10 @@ class gurobi_direct ( OptSolver ):
                 if 1 in obj_repn: # first-order terms
                     hash_to_variable_map = obj_repn[-1]
                     for var_hash, var_coefficient in iteritems(obj_repn[1]):
-                        label = self._symbol_map.getSymbol(hash_to_variable_map[var_hash], labeler)
+                        vardata = hash_to_variable_map[var_hash]
+                        self._referenced_variable_ids.add(id(vardata))
+                        label = self_symbol_map.getSymbol(vardata, labeler)
                         obj_expr.addTerms(var_coefficient, pyomo_gurobi_variable_map[label])
-                        # the coefficients are attached to the model when creating the
-                        # variables, below
 
                 if 2 in obj_repn:
                     obj_expr = QuadExpr(obj_expr)
@@ -257,15 +272,19 @@ class gurobi_direct ( OptSolver ):
                     for quad_repn, coef in iteritems(obj_repn[2]):
                         gurobi_expr = QuadExpr(coef)
                         for var_hash, exponent in iteritems(quad_repn):
-                            gurobi_var = pyomo_gurobi_variable_map[self._symbol_map.getSymbol(hash_to_variable_map[var_hash], labeler)]
-                            gurobi_expr *= gurobi_var 
+                            vardata = hash_to_variable_map[var_hash]
+                            self._referenced_variable_ids.add(id(vardata))
+                            gurobi_var = pyomo_gurobi_variable_map[self_symbol_map.getSymbol(vardata, labeler)]
+                            gurobi_expr *= gurobi_var
                             if exponent == 2:
                                 gurobi_expr *= gurobi_var
                         obj_expr += gurobi_expr
-            # need to cache the objective label, because the GUROBI python interface doesn't track this.
-            self._objective_label = self._symbol_map.getSymbol(obj_data, labeler)
 
-        grbmodel.setObjective(obj_expr,sense=sense)       
+            # need to cache the objective label, because the GUROBI python interface doesn't track this.
+            # _ObjectiveData objects will not be in the symbol map yet, so avoid some checks.
+            self._objective_label = self_symbol_map.createSymbol(obj_data, labeler)
+
+        grbmodel.setObjective(obj_expr, sense=sense)       
  
         # SOS constraints - largely taken from cpxlp.py so updates there,
         # should be applied here
@@ -275,37 +294,30 @@ class gurobi_direct ( OptSolver ):
         sos2 = self._capabilities.sos2
         
         modelSOS = ModelSOS()
-        for block in model.all_blocks():
-            for con in active_subcomponents_generator(block,SOSConstraint):
-                level = con.sos_level()
+        for block in pyomo_instance.all_blocks():
+            for soscondata in active_subcomponents_data_generator(block,SOSConstraint):
+                level = soscondata.get_level()
                 if (level == 1 and not sos1) or (level == 2 and not sos2) or (level > 2):
                     raise Exception("Solver does not support SOS level %s constraints" % (level,))
-                name = self._symbol_map.getSymbol( con, labeler )
-                masterIndex = con.sos_set_set()
-                if None in masterIndex:
-                    # A single constraint
-                    modelSOS.count_constraint(self._symbol_map,labeler,pyomo_gurobi_variable_map,con, name, level)
-                else:
-                    # A series of indexed constraints
-                    for index in masterIndex:
-                        modelSOS.count_constraint(self._symbol_map,labeler,pyomo_gurobi_variable_map,con, name, level, index)
+                modelSOS.count_constraint(self_symbol_map,labeler,pyomo_gurobi_variable_map,soscondata)
         
         if modelSOS.sosType:
             for key in modelSOS.sosType:
                 grbmodel.addSOS(modelSOS.sosType[key], \
                                 modelSOS.varnames[key], \
                                 modelSOS.weights[key] )
+                self._referenced_variable_ids.update(modelSOS.varids[key])
         
         grbmodel.update()
 
-        # Track the range constraints and there associated variables added by gurobi
+        # Track the range constraints and their associated variables added by gurobi
         self._last_native_var_idx = grbmodel.NumVars-1
         range_var_idx = grbmodel.NumVars
         _self_range_con_var_pairs = self._range_con_var_pairs = []
 
-        for block in model.all_blocks():
+        for block in pyomo_instance.all_blocks():
             
-            block_repn_suffix = block.subcomponent("repn")
+            block_repn_suffix = block.subcomponent("canonical_repn")
             if (block_repn_suffix is None) or (not block_repn_suffix.type() is Suffix) or (not block_repn_suffix.active is True):
                 block_repn_suffix = None
 
@@ -325,7 +337,8 @@ class gurobi_direct ( OptSolver ):
                         con_repn = block_repn_suffix.getValue(constraint_data)
 
                     offset = 0.0
-                    constraint_label = self._symbol_map.getSymbol(constraint_data, labeler)
+                    # _ConstraintData objects will not be in the symbol map yet, so avoid some checks.
+                    constraint_label = self_symbol_map.createSymbol(constraint_data, labeler)
 
                     if isinstance(con_repn, LinearCanonicalRepn):
 
@@ -342,8 +355,8 @@ class gurobi_direct ( OptSolver ):
 
                                 var_coefficient = con_repn.linear[i]
                                 var_value = con_repn.variables[i]
-    
-                                label = self._symbol_map.getSymbol(var_value, labeler)
+                                self._referenced_variable_ids.add(id(var_value))
+                                label = self_symbol_map.getSymbol(var_value, labeler)
                                 linear_coefs.append( var_coefficient )
                                 linear_vars.append( pyomo_gurobi_variable_map[label] )
     
@@ -363,7 +376,8 @@ class gurobi_direct ( OptSolver ):
                             hash_to_variable_map = con_repn[-1]
                             for var_hash, var_coefficient in iteritems(con_repn[1]):
                                 var = hash_to_variable_map[var_hash]
-                                label = self._symbol_map.getSymbol(var, labeler)
+                                self._referenced_variable_ids.add(id(var))
+                                label = self_symbol_map.getSymbol(var, labeler)
                                 linear_coefs.append( var_coefficient )
                                 linear_vars.append( pyomo_gurobi_variable_map[label] )
     
@@ -378,7 +392,9 @@ class gurobi_direct ( OptSolver ):
                             for quad_repn, coef in iteritems(con_repn[2]):
                                 gurobi_expr = QuadExpr(coef)
                                 for var_hash, exponent in iteritems(quad_repn):
-                                    gurobi_var = pyomo_gurobi_variable_map[self._symbol_map.getSymbol(hash_to_variable_map[var_hash], labeler)]
+                                    vardata = hash_to_variable_map[var_hash]
+                                    self._referenced_variable_ids.add(id(vardata))
+                                    gurobi_var = pyomo_gurobi_variable_map[self_symbol_map.getSymbol(vardata, labeler)]
                                     gurobi_expr *= gurobi_var 
                                     if exponent == 2:
                                         gurobi_expr *= gurobi_var
@@ -387,8 +403,10 @@ class gurobi_direct ( OptSolver ):
                     if constraint_data._equality:
                         sense = GRB.EQUAL    # Fixed
                         bound = constraint_data.lower()
-                        grbmodel.addConstr(
-                            lhs=expr, sense=sense, rhs=bound, name=constraint_label )
+                        grbmodel.addConstr(lhs=expr, 
+                                           sense=sense, 
+                                           rhs=bound, 
+                                           name=constraint_label )
                     else:
                         # L <= body <= U
                         if (constraint_data.upper is not None) and (constraint_data.lower is not None):
@@ -419,25 +437,75 @@ class gurobi_direct ( OptSolver ):
         grbmodel.update()
 
         self._gurobi_instance = grbmodel
+        self._pyomo_gurobi_variable_map = pyomo_gurobi_variable_map
 
     def warm_start_capable(self):
-        msg = "Gurobi has the ability to use warmstart solutions.  However, it "\
-              "has not yet been implemented into the Coopr gurobi_direct plugin."
-        logger.info( msg )
-        return False
 
+        return True
 
     def warm_start(self, instance):
-        pass
 
+        def _error_labeler(obj):
+            raise KeyError(
+                "Gurobi Warm Start Error: instance contains an object, %s, "
+                "that is not found in the symbol map.  Does the instance "
+                "match the provided symbol map?" % ( obj.cname(True), ))
+
+        from coopr.pyomo.base import Var
+
+        # the iteration order is identical to that used in generating
+        # the gurobi instance, so all should be well. ???
+        for block in instance.all_blocks():
+            for var_data in active_subcomponents_data_generator(block, Var):
+                if (var_data.stale is True) or (var_data.fixed is True):
+                    continue
+                pyomo_label = self._symbol_map.getSymbol(var_data, _error_labeler)
+                gurobi_variable = self._pyomo_gurobi_variable_map[pyomo_label]
+                gurobi_variable.setAttr(GRB.attr.Start, var_data.value)
+
+    def _presolve(self, *args, **kwds):
 
-    def _presolve(self, *args, **kwargs):
         from coopr.pyomo.base.PyomoModel import Model
 
-        self.suffixes = kwargs.pop( 'suffixes', self.suffixes )
-        self.warm_start_solve = kwargs.pop( 'warmstart', False )
-        self.keepfiles = kwargs.pop( 'keepfiles' , False )
-        self.tee = kwargs.pop( 'tee', False )
+        self.warm_start_solve = False
+        self.keepfiles = False
+        self.tee = False
+        self.symbolic_solver_labels = False
+        for key in kwds:
+            ### copied from base class _presolve
+            warn = False
+            if key == "logfile":
+                if kwds[key] is not None:
+                    warn = True
+            elif key == "solnfile":
+                if kwds[key] is not None:
+                    warn = True
+            elif key == "timelimit":
+                if kwds[key] is not None:
+                    warn = True
+            elif key == "tee":
+                self.tee=bool(kwds[key])
+            elif key == "options":
+                self.set_options(kwds[key])
+            elif key == "available":
+                self._assert_available=True
+            elif key == "symbolic_solver_labels":
+                self.symbolic_solver_labels = bool(kwds[key])
+            elif key == "output_fixed_variable_bounds":
+                if kwds[key] is not False:
+                    warn = True
+            elif key == "suffixes":
+                self.suffixes=kwds[key]
+            ###
+            elif key == 'keepfiles':
+                self.keepfiles = bool(kwds[key])
+            elif key == 'warmstart':
+                self.warm_start_solve = bool(kwds[key])
+            else:
+                raise ValueError("Unknown option="+key+" for solver="+self.type)
+
+            if warn is True:
+                logger.warn('"'+key+'" keyword ignored by solver='+self.type)
 
         model = args[0]
         if len(args) != 1:
@@ -455,7 +523,6 @@ class gurobi_direct ( OptSolver ):
             fname = self.options.write
             grbmodel.write( fname )
 
-        # Scaffolding in place
         if self.warm_start_solve is True:
 
             if len(args) != 1:
@@ -497,7 +564,7 @@ class gurobi_direct ( OptSolver ):
         # 'ResultFile', 'ImproveStartTime', 'ImproveStartGap', 'Threads', 'Dummy', 'OutputFlag']
         for key in self.options:
             prob.setParam( key, self.options[key] )
-            
+
         if 'relax_integrality' in self.options:
             for v in prob.getVars():
                 if v.vType != GRB.CONTINUOUS:
@@ -534,7 +601,6 @@ class gurobi_direct ( OptSolver ):
         elif GRB.NUMERIC         == status: return SolutionStatus.error
         raise RuntimeError('Unknown solution status returned by Gurobi solver')
 
-
     def _postsolve(self):
 
         # the only suffixes that we extract from GUROBI are
@@ -576,17 +642,75 @@ class gurobi_direct ( OptSolver ):
         problem = results.problem
         solver  = results.solver
 
+        # cache the variable and constraint dictionaries - otherwise,
+        # each invocation will include a lookup in a MapContainer, 
+        # which is extremely expensive.
+        soln_variables = soln.variable
+        soln_constraints = soln.constraint
+
         solver.name = "Gurobi %s.%s%s" % gurobi.version()
-        # solver.memory_used =
-        # solver.user_time = None
-        # solver.system_time = None
         solver.wallclock_time = gprob.Runtime
-        # solver.termination_condition = None
-        # solver.termination_message = None
+
+        if gprob.Status == 1: # problem is loaded, but no solution
+            solver.termination_message = "Model is loaded, but no solution information is available."
+            solver.termination_condition = TerminationCondition.other
+        if gprob.Status == 2: # optimal
+            solver.termination_message = "Model was solved to optimality (subject to tolerances)."
+            solver.termination_condition = TerminationCondition.optimal
+        elif gprob.Status == 3: # infeasible
+            solver.termination_message = "Model was proven to be infeasible."
+            solver.termination_condition = TerminationCondition.infeasible
+        elif gprob.Status == 4: # infeasible or unbounded
+            solver.termination_message = "Model was proven to be either infeasible or unbounded."
+            solver.termination_condition = TerminationCondition.infeasible # picking one of the pre-specified Coopr termination conditions - we don't have either-or.
+        elif gprob.Status == 5: # unbounded
+            solver.termination_message = "Model was proven to be unbounded."
+            solver.termination_condition = TerminationCondition.unbounded
+        elif gprob.Status == 6: # cutoff
+            solver.termination_message = "Optimal objective for model was proven to be worse than the value specified in the Cutoff parameter."
+            solver.termination_condition = TerminationCondition.minFunctionValue
+        elif gprob.Status == 7: # iteration limit
+            solver.termination_message = "Optimization terminated because the total number of simplex or barrier iterations exceeded specified limits."
+            solver.termination_condition = TerminationCondition.maxIterations
+        elif gprob.Status == 8: # node limit
+            solver.termination_message = "Optimization terminated because the total number of branch-and-cut nodes exceeded specified limits."
+            solver.termination_condition = TerminationCondition.maxEvaluations
+        elif gprob.Status == 9: # time limit
+            solver.termination_message = "Optimization terminated because the total time expended exceeded specified limits."
+            solver.termination_condition = TerminationCondition.maxTimeLimit
+        elif gprob.Status == 10: # solution limit
+            solver.termination_message = "Optimization terminated because the number of solutions found reached specified limits."
+            solver.termination_condition = TerminationCondition.other
+        elif gprob.Status == 11: # interrupted
+            solver.termination_message = "Optimization was terminated by the user."
+            solver.termination_condition = TerminationCondition.other
+        elif gprob.Status == 12: # numeric issues
+            solver.termination_message = "Optimization was terminated due to unrecoverable numerical difficulties."
+            solver.termination_condition = TerminationCondition.other
+        elif gprob.Status == 13: # suboptimal
+            solver.termination_message = "Optimization was unable to satisfy optimality tolerances; returned solution is sub-optimal."
+            solver.termination_condition = TerminationCondition.other
+        else: # unknown
+            solver.termination_message = "Unknown termination condition received following optimization."
+            solver.termination_condition = TerminationCondition.other
 
         problem.name = gprob.ModelName
-        problem.lower_bound = None
-        problem.upper_bound = None
+        if gprob.ModelSense > 0: # positive numbers indicate minimization
+            # depending on whether the problem is a MIP or an LP, and whether
+            # or not the solution is feasible, one or two of the attributes
+            # below may not exist, yielding an exception as a result of the query.
+            try:
+                problem.upper_bound = gprob.ObjVal
+                problem.lower_bound = gprob.ObjBound
+            except:
+                pass
+        else:
+            try:
+                problem.upper_bound = gprob.ObjBound
+                problem.lower_bound = gprob.ObjVal
+            except:
+                pass
+
         problem.number_of_constraints          = len(cons)+len(qcons)+gprob.NumSOS
         problem.number_of_nonzeros             = gprob.NumNZs
         problem.number_of_variables            = gprob.NumVars
@@ -602,41 +726,62 @@ class gurobi_direct ( OptSolver ):
         if problem.sense == GRB_MAX: problem.sense = ProblemSense.maximize
 
         soln.status = self._gurobi_get_solution_status()
+        soln.gap = None # until proven otherwise
 
         if soln.status in (SolutionStatus.optimal, SolutionStatus.stoppedByLimit):
+
             obj_val = gprob.ObjVal
+
+            if problem.number_of_binary_variables + problem.number_of_integer_variables > 0:
+                obj_bound = gprob.ObjBound
+
             if problem.sense == ProblemSense.minimize:
-                problem.lower_bound = obj_val
                 if problem.number_of_binary_variables + problem.number_of_integer_variables == 0:
                     problem.upper_bound = obj_val
+                else:
+                    problem.lower_bound = obj_bound
+                    problem.upper_bound = obj_val
             else:
-                problem.upper_bound = obj_val
+
                 if problem.number_of_binary_variables + problem.number_of_integer_variables == 0:
                     problem.lower_bound = obj_val
+                else:
+                    problem.upper_bound = obj_bound
+                    problem.lower_bound = obj_val
 
             soln.objective[self._objective_label].value = obj_val
+            if problem.number_of_binary_variables + problem.number_of_integer_variables == 0:
+                soln.gap = None
+            else:
+                soln.gap = math.fabs(obj_val - obj_bound)
             
             # Those variables not added by gurobi due to range constraints
             for var in itertools.islice(pvars,self._last_native_var_idx+1):
-                soln.variable[ var.VarName ] = {"Value" : var.X, "Id" : len(soln.variable) - 1}
+                soln_variables[ var.VarName ] = {"Value" : var.X, "Id" : len(soln_variables) - 1}
 
             if extract_reduced_costs is True:
                 for var in itertools.islice(pvars,self._last_native_var_idx+1):
-                    soln.variable[ var.VarName ]["Rc"] = var.Rc
+                    soln_variables[ var.VarName ]["Rc"] = var.Rc
+
+            if extract_duals or extract_slacks:
+                for con in cons:
+                    soln_constraints[con.ConstrName] = {"Id" : len(soln_constraints)}
+                for con in qcons:
+                    soln_constraints[con.QCName] = {"Id" : len(soln_constraints)}
 
             if extract_duals is True:
                 for con in cons:
                     # Pi attributes in Gurobi are the constraint duals
-                    soln.constraint[ con.ConstrName ].dual = con.Pi                
+                    soln_constraints[ con.ConstrName ]["Dual"] = con.Pi
                 for con in qcons:
                     # QCPI attributes in Gurobi are the constraint duals
-                    soln.constraint[ con.QCName ].dual = con.QCPi
+                    soln_constraints[ con.QCName ]["Dual"] = con.QCPi
 
             if extract_slacks is True:
                 for con in cons:
-                    soln.constraint[ con.ConstrName ].slack = con.Slack
+                    soln_constraints[ con.ConstrName ]["Slack"] = con.Slack
                 for con in qcons:
-                    soln.constraint[ con.QCName ].slack = con.QCSlack
+                    soln_constraints[ con.QCName ]["Slack"] = con.QCSlack
                 # The above loops may include range constraints but will 
                 # always report a slack of zero since gurobi transforms
                 # range constraints by adding a slack variable in the following way
@@ -657,11 +802,15 @@ class gurobi_direct ( OptSolver ):
                     # f(x)-L
                     Ls_ = var.UB-var.X
                     if Us_ > Ls_:
-                        soln.constraint[ con.ConstrName ].slack = Us_
+                        soln_constraints[ con.ConstrName ]["Slack"] = Us_
                     else:
-                        soln.constraint[ con.ConstrName ].slack = -Ls_
-                    
-
+                        soln_constraints[ con.ConstrName ]["Slack"] = -Ls_
+            
+            byObject = self._symbol_map.byObject
+            referenced_varnames = set(byObject[varid] for varid in self._referenced_variable_ids)
+            names_to_delete = set(soln_variables.keys())-referenced_varnames
+            for varname in names_to_delete:
+                del soln_variables[varname]
 
         results.solution.insert(soln)
 
@@ -670,7 +819,6 @@ class gurobi_direct ( OptSolver ):
         # Done with the model object; free up some memory.
         self._last_native_var_idx = -1
         self._range_con_var_pairs = []
-        del gprob, self._gurobi_instance
 
         # let the base class deal with returning results.
         return OptSolver._postsolve(self)
