@@ -19,7 +19,7 @@ from coopr.opt.blackbox import problem
 class PatternSearch(solver.COLINSolver):
     """A pattern search optimizer"""
 
-    pyutilib.component.core.alias('ps', doc='Python interface to a simple pattern search optimizer')
+    pyutilib.component.core.alias('ps', doc='Coopr\'s simple pattern search optimizer')
 
     def __init__(self, **kwds):
         """ Constructor """
