@@ -11,7 +11,7 @@
 import pyutilib.component.core
 pyutilib.component.core.PluginGlobals.push_env( 'pyutilib.autotest' )
 
-import mip
-import pyomo
+import coopr.plugins.testdriver.mip
+import coopr.plugins.testdriver.pyomo
 
 pyutilib.component.core.PluginGlobals.pop_env()
