@@ -23,9 +23,9 @@ from coopr.opt import ResultsFormat, ProblemFormat, ConverterError
 pyutilib.services.register_executable('cplexamp')
 coopr.opt.SolverResults.default_print_options.ignore_time = True
 try:
-    cplex = coopr.plugins.solvers.CPLEX(keepFiles=True)
+    cplex = coopr.plugins.solvers.CPLEX(keepfiles=True)
     cplex_available = (not cplex.executable() is None) and cplex.available(False)
-    asl = coopr.plugins.solvers.ASL(keepFiles=True, options={'solver':'cplexamp'})
+    asl = coopr.plugins.solvers.ASL(keepfiles=True, options={'solver':'cplexamp'})
     cplexamp_available= cplex_available and (not asl.executable() is None) and asl.available(False)
 except pyutilib.common.ApplicationError:
     cplexamp_available=False
@@ -52,9 +52,9 @@ class mock_all(unittest.TestCase):
         pyutilib.services.TempfileManager.sequential_files(0)
         pyutilib.services.TempfileManager.tempdir = currdir
         if flag:
-            self.asl = coopr.plugins.solvers.ASL(keepFiles=True, options={'solver':'cplexamp'})
+            self.asl = coopr.plugins.solvers.ASL(keepfiles=True, options={'solver':'cplexamp'})
         else:
-            self.asl = coopr.plugins.solvers.MockASL(keepFiles=True, options={'solver':'cplexamp'})
+            self.asl = coopr.plugins.solvers.MockASL(keepfiles=True, options={'solver':'cplexamp'})
         self.asl.suffixes=['.*']
 
     def tearDown(self):
@@ -91,7 +91,7 @@ class mock_all(unittest.TestCase):
 
     def Xtest_solve3(self):
         """ Test ASL - test2.lp """
-        results = self.asl.solve(currdir+"test2.lp", logfile=currdir+"test_solve3.log", keepFiles=True)
+        results = self.asl.solve(currdir+"test2.lp", logfile=currdir+"test_solve3.log", keepfiles=True)
         results.write(filename=currdir+"test_solve3.txt", times=False, format='json')
         self.assertMatchesJsonBaseline(currdir+"test_solve3.txt", currdir+"test2_asl.txt")
         if os.path.exists(currdir+"test2.solution.dat"):
@@ -100,15 +100,9 @@ class mock_all(unittest.TestCase):
 
     def test_solve4(self):
         """ Test ASL - test4.nl """
-        if pico_convert_available:
-            results = self.asl.solve(currdir+"test4.nl", logfile=currdir+"test_solve4.log")
-            results.write(filename=currdir+"test_solve4.txt", times=False, format='json')
-            self.assertMatchesJsonBaseline(currdir+"test_solve4.txt", currdir+"test4_asl.txt", tolerance=1e-4)
-        else:
-            try:
-                results = self.asl.solve(currdir+"test4.nl", logfile=currdir+"test_solve4.log")
-            except ConverterError:
-                return
+        results = self.asl.solve(currdir+"test4.nl", logfile=currdir+"test_solve4.log")
+        results.write(filename=currdir+"test_solve4.txt", times=False, format='json')
+        self.assertMatchesJsonBaseline(currdir+"test_solve4.txt", currdir+"test4_asl.txt", tolerance=1e-4)
         #os.remove(currdir+"test4.sol")
         #os.remove(currdir+"test_solve4.log")
 
