@@ -36,7 +36,7 @@ class MockArg(object):
     def valid_problem_types(self):
         return [ProblemFormat.pyomo]
 
-    def write(self,filename="", format=None, solver_capability=None, symbolic_solver_labels=False):
+    def write(self,filename="", format=None, solver_capability=None, io_options={}):
         return (filename,None)
 
 class MockArg2(MockArg):
@@ -44,7 +44,7 @@ class MockArg2(MockArg):
     def valid_problem_types(self):
         return [ProblemFormat.nl]
 
-    def write(self,filename="", format=None, solver_capability=None, symbolic_solver_labels=False):
+    def write(self,filename="", format=None, solver_capability=None, io_options={}):
         OUTPUT=open(filename,"w")
         INPUT=open(currdir+"test4.nl")
         for line in INPUT:
@@ -58,12 +58,12 @@ class MockArg3(MockArg):
     def valid_problem_types(self):
         return [ProblemFormat.mod]
 
-    def write(self,filename="", format=None, solver_capability=None, symbolic_solver_labels=False):
+    def write(self,filename="", format=None, solver_capability=None, io_options={}):
         return (filename,None)
 
 class MockArg4(MockArg):
 
-    def write(self,filename="", format=None, solver_capability=None, symbolic_solver_labels=False):
+    def write(self,filename="", format=None, solver_capability=None, io_options={}):
         OUTPUT=open(filename,"w")
         INPUT=open(currdir+"test4.nl")
         for line in INPUT:
