@@ -68,7 +68,7 @@ class OptFactoryDebug(unittest.TestCase):
         Testing the coopr.opt solver factory with MIP solvers
         """
         ans = sorted(coopr.opt.SolverFactory.services())
-        tmp = ['_mock_asl', '_mock_cbc', '_mock_cplex', '_mock_glpk', '_mock_pico', 'cbc', 'cplex', 'glpk', 'pico', 'stest3', '_asl']
+        tmp = ['_mock_asl', '_mock_cbc', '_mock_cplex', '_mock_glpk', '_mock_pico', 'cbc', 'cplex', 'glpk', 'pico', 'stest3', 'asl']
         tmp.sort()
         self.assertTrue(set(tmp) <= set(ans), msg="Set %s is not a subset of set %s" %(tmp,ans))
 
