@@ -1,13 +1,17 @@
 import os
-import pyutilib.th as unittest
+import pickle
+import sys
 from nose.tools import timed
+
+from pyutilib.common import ApplicationError
+import pyutilib.th as unittest
 from pyutilib.misc.pyyaml_util import *
+from pyutilib.misc import Options
+
 import coopr.pyomo.scripting.util as util
 import coopr.plugins
 from coopr.pyomo.scripting.util import apply_optimizer
-from pyutilib.misc import Options
-import pickle
-import sys
+
 currdir = os.path.dirname(os.path.abspath(__file__))
 sys.path.append(currdir)
 problem_list = ['trivial_model']
@@ -25,61 +29,62 @@ def has_python(name):
 
 def has_gurobi_lp():
     try:
-        gurobi = coopr.plugins.solvers.GUROBI(keepFiles=True)
+        gurobi = coopr.plugins.solvers.GUROBI(keepfiles=True)
         available = (not gurobi.executable() is None) and gurobi.available(False)
         return available
-    except pyutilib.common.ApplicationError:
+    except ApplicationError:
+        raise
         return False
 def has_cplex_lp():
     try:
-        cplex = coopr.plugins.solvers.CPLEX(keepFiles=True)
+        cplex = coopr.plugins.solvers.CPLEX(keepfiles=True)
         available = (not cplex.executable() is None) and cplex.available(False)
         return available
-    except pyutilib.common.ApplicationError:
+    except ApplicationError:
         return False
 def has_glpk_lp():
     try:
-        glpk = coopr.plugins.solvers.GLPK(keepFiles=True)
+        glpk = coopr.plugins.solvers.GLPK(keepfiles=True)
         available = (not glpk.executable() is None) and glpk.available(False)
         return available
-    except pyutilib.common.ApplicationError:
+    except ApplicationError:
         return False
 def has_cbc_lp():
     try:
-        cbc = coopr.plugins.solvers.CBC(keepFiles=True)
+        cbc = coopr.plugins.solvers.CBC(keepfiles=True)
         available = (not cbc.executable() is None) and cbc.available(False)
         return available
-    except pyutilib.common.ApplicationError:
+    except ApplicationError:
         return False
 def has_pico_lp():
     try:
-        PICO = coopr.plugins.solvers.PICO(keepFiles=True)
+        PICO = coopr.plugins.solvers.PICO(keepfiles=True)
         available = (not PICO.executable() is None) and PICO.available(False)
         return available
-    except pyutilib.common.ApplicationError:
+    except ApplicationError:
         return False
 
 def has_cplex_nl():
     try:
-        cplex = coopr.plugins.solvers.CPLEX(keepFiles=True)
+        cplex = coopr.plugins.solvers.CPLEX(keepfiles=True)
         available = (not cplex.executable() is None) and cplex.available(False)
-        asl = coopr.plugins.solvers.ASL(keepFiles=True, options={'solver':'cplexamp'})
+        asl = coopr.plugins.solvers.ASL(keepfiles=True, options={'solver':'cplexamp'})
         return available and (not asl.executable() is None) and asl.available(False)
-    except pyutilib.common.ApplicationError:
+    except ApplicationError:
         return False
 def has_gurobi_nl():
     try:
-        gurobi = coopr.plugins.solvers.GUROBI(keepFiles=True)
+        gurobi = coopr.plugins.solvers.GUROBI(keepfiles=True)
         available = (not gurobi.executable() is None) and gurobi.available(False)
-        asl = coopr.plugins.solvers.ASL(keepFiles=True, options={'solver':'gurobi_ampl'})
+        asl = coopr.plugins.solvers.ASL(keepfiles=True, options={'solver':'gurobi_ampl'})
         return available and (not asl.executable() is None) and asl.available(False)
-    except pyutilib.common.ApplicationError:
+    except ApplicationError:
         return False
 def has_ipopt_nl():
     try:
-        asl = coopr.plugins.solvers.ASL(keepFiles=True, options={'solver':'ipopt'})
+        asl = coopr.plugins.solvers.ASL(keepfiles=True, options={'solver':'ipopt'})
         return (not asl.executable() is None) and asl.available(False)
-    except pyutilib.common.ApplicationError:
+    except ApplicationError:
         return False
 
 writer_solver = []
@@ -148,7 +153,7 @@ def createTestMethod(pName,pickle_which,problem,solver,writer,do_test):
 
         opt_data = apply_optimizer(data,instance=inst)
 
-        f = open('junk.pickle','w')
+        f = open('junk.pickle','wb')
         if pickle_which == 'instance':
             inst.load(opt_data.results)
             pickle.dump(inst,f)
